SENTRY_SDK_URL = (
    "https://0da75bab4671455ea1b7580cb93649f5@o444286.ingest.sentry.io/" "5645833"
)

SERVER_COMPAT_ERROR_MSG = """
The server you're connecting to is out-of-date.
If you know the server admin, please ask them to upgrade the server.
 * Reported server version: \t{}
 * Compatible version: \t\t{}

To prevent data corruption, shippy will not work with an outdated server. Exiting...
"""

SHIPPY_COMPAT_ERROR_MSG = """
Error: shippy is out-of-date and will not with this server instance.
Upgrade shippy with the following command:
\tpip3 install --upgrade shipper-shippy

Version information:
 * Reported compatible version by server: \t{}
 * Your current shippy version: \t\t{}
"""

SHIPPY_OUTDATED_MSG = """
Warning: shippy is out-of-date.
 * Current version: \t{}
 * New version: \t{}

We recommend updating shippy with the following command:
\tpip3 install --upgrade shipper-shippy
"""

RATE_LIMIT_WAIT_STATUS_MSG = (
    "Waiting to resume after being rate limited. shippy will "
    "resume uploading in {} seconds."
)


CANNOT_CONTACT_SERVER_ERROR_MSG = "Cannot contact the server. "
UNEXPECTED_SERVER_RESPONSE_ERROR_MSG = "The server returned an unexpected response. "
FAILED_TO_RETRIEVE_SERVER_VERSION_ERROR_MSG = (
    "Failed to retrieve server version information! "
)
FAILED_TO_LOG_IN_ERROR_MSG = "Failed to log into server! "

UNHANDLED_EXCEPTION_MSG = """
shippy crashed for an unknown reason. :(
To figure out what went wrong, please pass along the full output.
----
URL of request: {}
Request response code: {}
Request response: {}
---
"""



import hashlib
import os.path
import requests

from json import JSONDecodeError
from rich.console import Console
from rich.progress import (
    BarColumn,
    DownloadColumn,
    Progress,
    TextColumn,
    TimeRemainingColumn,
    TransferSpeedColumn,
)

from .config import get_config_value, set_config_value
from .constants import (
    UNHANDLED_EXCEPTION_MSG,
    FAILED_TO_RETRIEVE_SERVER_VERSION_ERROR_MSG,
    CANNOT_CONTACT_SERVER_ERROR_MSG,
    FAILED_TO_LOG_IN_ERROR_MSG,
    UNEXPECTED_SERVER_RESPONSE_ERROR_MSG,
    RATE_LIMIT_WAIT_STATUS_MSG,
)
from .exceptions import LoginException, UploadException
from .helper import print_error

console = Console()

# Set up progress bar
progress = Progress(
    TextColumn("[progress.description]{task.description}"),
    BarColumn(),
    "[progress.percentage]{task.percentage:>3.1f}%",
    "•",
    DownloadColumn(),
    "•",
    TransferSpeedColumn(),
    "•",
    TimeRemainingColumn(),
    transient=True,
)


def handle_undefined_response(request):
    """Handles undefined responses sent back by the server"""
    try:
        raise Exception(
            UNHANDLED_EXCEPTION_MSG.format(
                request.url, request.status_code, request.json()
            )
        )
    except JSONDecodeError:
        raise Exception(
            UNHANDLED_EXCEPTION_MSG.format(
                request.url, request.status_code, request.content
            )
        )


def get_server_version_info(server_url):
    """Gets server version information"""
    version_url = f"{server_url}/api/v1/system/info/"
    try:
        r = requests.get(version_url)
        if r.status_code == 200:
            return r.json()
        else:
            print_error(
                msg=FAILED_TO_RETRIEVE_SERVER_VERSION_ERROR_MSG,
                newline=True,
                exit_after=True,
            )
    except requests.exceptions.RequestException:
        print_error(
            msg=CANNOT_CONTACT_SERVER_ERROR_MSG
            + FAILED_TO_RETRIEVE_SERVER_VERSION_ERROR_MSG,
            newline=True,
            exit_after=True,
        )


def login_to_server(username, password, server_url):
    """Authenticates to server and returns authorization token"""
    login_url = f"{server_url}/api/v1/maintainers/login/"
    try:
        r = requests.post(login_url, data={"username": username, "password": password})

        if r.status_code == 200:
            data = r.json()
            return data["token"]
        elif r.status_code == 400 and r.json()["error"] == "blank_username_or_password":
            raise LoginException("Username or password must not be blank.")
        elif r.status_code == 404 and r.json()["error"] == "invalid_credential":
            raise LoginException("Invalid credentials!")
        # Really, really weird edge case where HTTP URLs would redirect and cause a
        # GET request
        elif (
            r.status_code == 405
            and r.json()["detail"] == 'Method "GET" not allowed.'
            and server_url[0:5] == "http:"
        ):
            print(
                "It seems like you entered a HTTP address when setting up shippy, but "
                "the server instance uses HTTPS. shippy automatically corrected your "
                "server URL in the configuration file."
            )
            server_url = f"https://{server_url[7:]}"
            set_config_value("shippy", "server", server_url)
            # Attempt logging in again
            return login_to_server(username, password, server_url)

        # Returned status code matches no scenario, abort
        handle_undefined_response(r)
    except LoginException as e:
        raise e
    except JSONDecodeError:
        print_error(
            msg=UNEXPECTED_SERVER_RESPONSE_ERROR_MSG + FAILED_TO_LOG_IN_ERROR_MSG,
            newline=True,
            exit_after=True,
        )
    except requests.exceptions.RequestException:
        print_error(
            msg=CANNOT_CONTACT_SERVER_ERROR_MSG + FAILED_TO_LOG_IN_ERROR_MSG,
            newline=True,
            exit_after=True,
        )


def check_token(server_url, token):
    token_check_url = f"{server_url}/api/v1/maintainers/token_check/"
    r = requests.get(token_check_url, headers={"Authorization": f"Token {token}"})

    if r.status_code == 200:
        print(f"Successfully validated token! Hello, {r.json()['username']}.")
        return True
    return False


def get_regex_pattern(server_url, token):
    regex_pattern_url = f"{server_url}/api/v1/maintainers/upload_filename_regex_pattern"
    r = requests.get(regex_pattern_url, headers={"Authorization": f"Token {token}"})

    if r.status_code == 200:
        return r.json()["pattern"]


def upload(server_url, build_file_path, token):
    """Upload given build files to specified server with token"""
    upload_url = f"{server_url}/api/v1/maintainers/chunked_upload/"

    chunk_size = 1000000  # 1 MB
    current_byte = 0
    total_file_size = os.path.getsize(build_file_path)

    with progress:
        upload_progress = progress.add_task(
            "[green]Uploading...", total=total_file_size
        )

        with open(build_file_path, "rb") as build_file:
            chunk_data = build_file.read(chunk_size)
            while chunk_data:
                try:
                    chunk_request = requests.put(
                        upload_url,
                        headers={
                            "Authorization": f"Token {token}",
                            "Content-Range": (
                                f"bytes {current_byte}-"
                                f"{current_byte + len(chunk_data) - 1}/{total_file_size}"
                            ),
                        },
                        data={"filename": build_file_path},
                        files={"file": chunk_data},
                    )

                    if chunk_request.status_code == 200:
                        upload_url = (
                            f"{server_url}/api/v1/maintainers/chunked_upload/"
                            f"{chunk_request.json()['id']}/"
                        )
                        current_byte += len(chunk_data)
                        progress.update(upload_progress, completed=current_byte)

                        # Read next chunk and continue
                        chunk_data = build_file.read(chunk_size)
                    elif chunk_request.status_code == 429:
                        print("shippy has been rate-limited.")
                        import re

                        wait_rate_limit(
                            int(re.findall(r"\d+", chunk_request.json()["detail"])[0])
                        )
                    elif int(chunk_request.status_code / 100) == 4:
                        try:
                            response_json = chunk_request.json()
                            raise UploadException(response_json["message"])
                        except KeyError:
                            raise UploadException(response_json)
                        except JSONDecodeError:
                            raise UploadException(
                                "Something went wrong during the upload."
                            )
                    else:
                        raise UploadException(
                            "An unidentified error occurred starting the upload."
                        )
                except requests.exceptions.RequestException:
                    raise UploadException(
                        "Something went wrong during the upload and the connection to "
                        "the server was lost!"
                    )

    # Finalize upload to begin processing
    try:
        with console.status(
            "Waiting for the server to process the uploaded build. This may take "
            "around 30 seconds... "
        ):
            # Check which hash we need to send over
            server_requests_checksum_type = get_server_version_info(
                server_url=server_url
            )["shippy_upload_checksum_type"]
            checksum_value = get_hash_of_file(
                build_file_path, checksum_type=server_requests_checksum_type
            )
            finalize_request = requests.post(
                upload_url,
                headers={"Authorization": f"Token {token}"},
                data={server_requests_checksum_type: checksum_value},
            )

            upload_exception_check(finalize_request, build_file_path)

            # Check if build should be disabled immediately after run
            check_build_disable(server_url, token, finalize_request.json()["build_id"])
    except UploadException as e:
        raise e
    except requests.exceptions.RequestException:
        raise UploadException(
            "Something went wrong during the upload and the connection to the server "
            "was lost!"
        )


def get_hash_of_file(filename, checksum_type):
    if checksum_type.lower() == "md5":
        hash_obj = hashlib.md5()
    elif checksum_type.lower() == "sha256":
        hash_obj = hashlib.sha256()
    else:
        # Unsupported checksum type
        return None

    with open(filename, "rb") as file:
        content = file.read()
        hash_obj.update(content)
    return hash_obj.hexdigest()


def get_hash_from_checksum_file(checksum_file):
    with open(checksum_file, "r") as checksum_file_raw:
        line = checksum_file_raw.readline()
        values = line.split(" ")
        return values[0]


def find_checksum_file(filename):
    valid_checksum_types = ["md5", "sha256"]
    has_checksum_file_type = None
    has_sum_postfix = False
    for checksum_type in valid_checksum_types:
        if os.path.isfile(f"{filename}.{checksum_type}"):
            has_checksum_file_type = checksum_type
            has_sum_postfix = False
        elif os.path.isfile(f"{filename}.{checksum_type}sum"):
            has_checksum_file_type = checksum_type
            has_sum_postfix = True
    return has_checksum_file_type, has_sum_postfix


def wait_rate_limit(s):
    import time

    with console.status(RATE_LIMIT_WAIT_STATUS_MSG.format(s)) as status:
        while s:
            time.sleep(1)
            s -= 1
            status.update(status=RATE_LIMIT_WAIT_STATUS_MSG.format(s))


def upload_exception_check(request, build_file):
    if request.status_code == 200:
        print(f"Successfully uploaded the build {build_file}!")
        return
    elif int(request.status_code / 100) == 4:
        try:
            response_json = request.json()
            raise UploadException(response_json["message"])
        except JSONDecodeError:
            raise UploadException("An unknown error occurred parsing the response.")
    elif int(request.status_code / 100) == 5:
        raise UploadException(
            "An internal server error occurred. Please contact the admins."
        )

    handle_undefined_response(request)


def check_build_disable(server_url, token, build_id):
    try:
        disable_build_on_upload = (
            get_config_value("shippy", "DisableBuildOnUpload") == "true"
        )
    except KeyError:
        # Not defined
        return

    if disable_build_on_upload:
        disable_build_url = (
            f"{server_url}/api/v1/maintainers/build/enabled_status_modify/"
        )
        r = requests.post(
            disable_build_url,
            headers={"Authorization": f"Token {token}"},
            data={"build_id": build_id, "enable": False},
        )

        if r.status_code == 200:
            print("Build has been automatically disabled, following configuration.")
            print("If this is unexpected, please check your configuration.")
        else:
            print("There was a problem disabling the build.")



import argparse
import re
import glob
import os.path

import requests
import semver
import sentry_sdk

from rich import print
from rich.console import Console

from .client import (
    login_to_server,
    upload,
    get_server_version_info,
    get_regex_pattern,
    get_hash_from_checksum_file,
    check_token,
    get_hash_of_file,
    find_checksum_file,
)
from .config import get_config_value, set_config_value, get_optional_true_config_value
from .constants import (
    SENTRY_SDK_URL,
    SERVER_COMPAT_ERROR_MSG,
    SHIPPY_COMPAT_ERROR_MSG,
    SHIPPY_OUTDATED_MSG,
)
from .exceptions import LoginException, UploadException
from .helper import input_yn, print_error, print_warning, print_success
from .version import __version__, server_compat_version

ignore_errors = [KeyboardInterrupt, ConnectionError]

sentry_sdk.init(
    SENTRY_SDK_URL,
    traces_sample_rate=1.0,
    release=__version__,
    ignore_errors=ignore_errors,
)

console = Console()


def main():
    print(f"Welcome to shippy (v.{__version__})!")

    # Check if we cannot prompt the user (default to auto-upload)
    upload_without_prompt = get_optional_true_config_value(
        "shippy", "UploadWithoutPrompt"
    )

    # Get commandline arguments
    args = init_argparse()
    upload_without_prompt = upload_without_prompt or args.yes

    # Check for updates
    check_shippy_update()

    # Check if server config is valid
    try:
        server_url = get_config_value("shippy", "server")
        if not check_server_url_schema(server_url):
            print_error(
                msg=f"The configuration file is corrupt. Please delete it and restart shippy.",
                newline=True,
                exit_after=True,
            )
        
        token = get_config_value("shippy", "token")

        token = check_token_validity(server_url, token)
    except KeyError:
        print_warning(
            "No configuration file found or configuration is invalid. You need to "
            "configure shippy before you can start using it."
        )
        server_url = get_server_url()
        token = get_token(server_url)

        # In case login function updated server URL, we need to fetch it again
        server_url = get_config_value("shippy", "server")

    # Check server version compatibility
    check_server_compat(server_url)

    # Get regex pattern from server
    regex_pattern = get_regex_pattern(server_url=server_url, token=token)

    # Search current directory for files
    builds = get_builds_in_current_dir(regex_pattern)

    if len(builds) == 0:
        print_error(
            msg="No files matching the submission criteria were detected in the "
            "current directory.",
            newline=True,
            exit_after=False,
        )
    else:
        print(f"Detected {len(builds)} build(s):")
        for build in builds:
            print(f"\t{build}")

        if not upload_without_prompt and len(builds) > 1:
            print_warning("You seem to be uploading multiple builds. ", newline=False)
            if not input_yn("Are you sure you want to continue?", default=False):
                return

        for build in builds:
            # Check build file validity
            if not check_build(build):
                print_warning("Invalid build. Skipping...")
                continue

            if upload_without_prompt or input_yn(f"Uploading build {build}. Start?"):
                try:
                    upload(server_url=server_url, build_file_path=build, token=token)
                except UploadException as exception:
                    print_error(exception, newline=True, exit_after=False)


def init_argparse():
    parser = argparse.ArgumentParser(
        description="Client-side tool for interfacing with shipper"
    )
    parser.add_argument(
        "-y",
        "--yes",
        action="store_true",
        help="Upload builds automatically without prompting",
    )
    return parser.parse_args()


def check_server_compat(server_url):
    with console.status(
        "Please wait while shippy contacts the remote server to check compatibility... "
    ):
        server_version_info = get_server_version_info(server_url)

    # Check if shipper version is compatible
    if semver.compare(server_version_info["version"], server_compat_version) == -1:
        print_error(
            msg=SERVER_COMPAT_ERROR_MSG.format(
                server_version_info["version"], server_compat_version
            ),
            newline=True,
            exit_after=True,
        )

    # Check if shippy version is compatible
    if semver.compare(server_version_info["shippy_compat_version"], __version__) == 1:
        print_error(
            msg=SHIPPY_COMPAT_ERROR_MSG.format(
                server_version_info["shippy_compat_version"], __version__
            ),
            newline=True,
            exit_after=True,
        )

    print_success("Finished compatibility check. No problems found.")


def check_token_validity(server_url, token):
    with console.status(
        "Please wait while shippy contacts the remote server to check if the token is "
        "still valid... "
    ):
        is_token_valid = check_token(server_url, token)

    if not is_token_valid:
        # Token check failed, prompt for login again
        print_warning("The saved token is invalid. Please sign-in again.")
        token = get_token(server_url)
    return token


def check_shippy_update():
    with console.status("Please wait while shippy checks for updates... "):
        r = requests.get(
            "https://api.github.com/repos/shipperstack/shippy/releases/latest"
        )
        latest_version = r.json()["name"]

    if semver.compare(__version__, latest_version) == -1:
        print(SHIPPY_OUTDATED_MSG.format(__version__, latest_version))
    else:
        print_success("Finished update check. shippy is up-to-date!")


def get_builds_in_current_dir(regex_pattern):
    with console.status("Detecting builds in current directory..."):
        builds = []
        files = [f for f in glob.glob("*.zip")]
        for file in files:
            if re.search(regex_pattern, file):
                builds.append(file)

        return builds


def check_build(filename):
    """Makes sure the build is valid"""
    print(f"Validating build {filename}...")

    # Validate that there is a matching checksum file
    has_checksum_file_type, has_sum_postfix = find_checksum_file(filename=filename)

    if has_checksum_file_type is None:
        print_warning(
            "This build does not have a matching checksum file. ", newline=False
        )
        return False

    # Validate checksum
    with console.status(
        f"Checking {has_checksum_file_type.upper()} hash of {filename}... this may "
        "take a couple of seconds. "
    ):
        hash_val = get_hash_of_file(
            filename=filename, checksum_type=has_checksum_file_type
        )
        if not has_sum_postfix:
            actual_hash_val = get_hash_from_checksum_file(
                f"{filename}.{has_checksum_file_type}"
            )
        else:
            actual_hash_val = get_hash_from_checksum_file(
                f"{filename}.{has_checksum_file_type}sum"
            )
        if hash_val != actual_hash_val:
            print_error(
                msg="This build's checksum is invalid. ",
                newline=False,
                exit_after=False,
            )
            return False
        print_success(f"{has_checksum_file_type.upper()} hash of {filename} matched.")

    build_slug, _ = os.path.splitext(filename)
    _, _, _, build_type, build_variant, _ = build_slug.split("-")

    # Check build type
    if build_type != "OFFICIAL":
        print_error(msg="This build is not official. ", newline=False, exit_after=False)
        return False

    # Check build variant
    valid_variants = ["gapps", "vanilla", "foss", "goapps"]
    if build_variant not in valid_variants:
        print_error(
            msg="This build has an unknown variant. ", newline=False, exit_after=False
        )
        return False

    print_success(f"Validation of build {filename} complete. No problems found.")
    return True


def get_server_url():
    try:
        while True:
            server_url = input("Enter the server URL: ")
            if not check_server_url_schema(server_url):
                # noinspection HttpUrlsUsage
                print_error(
                    msg="Server URL is missing either http:// or https://.",
                    newline=True,
                    exit_after=False,
                )
            else:
                break

        if server_url[-1] == "/":
            server_url = server_url[:-1]

        set_config_value("shippy", "server", server_url)

        return server_url
    except KeyboardInterrupt:
        exit(0)


def check_server_url_schema(url):
    return "http" in url


def get_token(server_url):
    while True:
        from getpass import getpass

        try:
            username = input("Enter your username: ")
            password = getpass(prompt="Enter your password: ")

            try:
                token = login_to_server(username, password, server_url)
                set_config_value("shippy", "token", token)
                return token
            except LoginException as exception:
                print_error(
                    f"{exception} Please try again.", newline=True, exit_after=False
                )
        except KeyboardInterrupt:
            exit(0)


if __name__ == "__main__":
    main()



import configparser

from pathlib import Path

home_dir = str(Path.home())

# Constants
CONFIGURATION_FILE = f"{home_dir}/.shippy.ini"

# Load configuration
config = configparser.ConfigParser()
config.optionxform = str
config.read(CONFIGURATION_FILE)


def get_config_value(section, key):
    return config[section][key]


def get_optional_true_config_value(section, key):
    try:
        value = config[section][key] == "true"
        return value
    except KeyError:
        # Set default to false so users can change it later
        set_config_value(section, key, "false")
        return False


def set_config_value(section, key, value):
    config_init()
    config[section][key] = value
    config_save()


def delete_deprecated_config():
    if config.has_section("shipper"):
        config.remove_section("shipper")


def config_init():
    if not config.has_section("shippy"):
        config.add_section("shippy")
    delete_deprecated_config()


def config_save():
    with open(CONFIGURATION_FILE, "w+") as config_file:
        config.write(config_file)



from setuptools import setup, find_packages
from distutils.util import convert_path

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

# Read version file
main_ns = {}
ver_path = convert_path("shippy/version.py")
with open(ver_path) as ver_file:
    exec(ver_file.read(), main_ns)

setup(
    name="shipper-shippy",
    version=main_ns["__version__"],
    author="Eric Park",
    author_email="me@ericswpark.com",
    description="Client-side tool to interface with shipper",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ericswpark/shippy",
    project_urls={
        "Bug Tracker": "https://github.com/ericswpark/shippy/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(exclude=("tests",)),
    python_requires=">=3.7",
    install_requires=[
        "requests",
        "semver",
        "sentry-sdk",
        "humanize",
        "rich",
    ],
    entry_points={
        "console_scripts": [
            "shippy=shippy.__main__:main",
        ]
    },
    include_package_data=False,
)
